#include "stdutils.h"

#ifndef _MPU6050_H
#define _MPU6050_H

// See also MPU-6050 Register Map and Descriptions, Revision 4.0, RM-MPU-6050A-00, Rev. 1.4, 9/9/2013 for registers not listed in 
// above document; the MPU6050 and MPU9150 are virtually identical but the latter has a different register map

#define SELF_TEST_X_GYRO 0x00                  
#define SELF_TEST_Y_GYRO 0x01                                                                          
#define SELF_TEST_Z_GYRO 0x02
#define SELF_TEST_X_ACCEL 0x0D
#define SELF_TEST_Y_ACCEL 0x0E    
#define SELF_TEST_Z_ACCEL 0x0F

#define SELF_TEST_A      0x10

#define XG_OFFSET_H      0x13  // User-defined trim values for gyroscope
#define XG_OFFSET_L      0x14
#define YG_OFFSET_H      0x15
#define YG_OFFSET_L      0x16
#define ZG_OFFSET_H      0x17
#define ZG_OFFSET_L      0x18
#define SMPLRT_DIV       0x19
#define CONFIG           0x1A
#define GYRO_CONFIG      0x1B
#define ACCEL_CONFIG     0x1C
#define ACCEL_CONFIG2    0x1D
#define LP_ACCEL_ODR     0x1E   
#define WOM_THR          0x1F   

#define MOT_DUR          0x20  // Duration counter threshold for motion interrupt generation, 1 kHz rate, LSB = 1 ms
#define ZMOT_THR         0x21  // Zero-motion detection threshold bits [7:0]
#define ZRMOT_DUR        0x22  // Duration counter threshold for zero motion interrupt generation, 16 Hz rate, LSB = 64 ms

#define FIFO_EN          0x23
#define I2C_MST_CTRL     0x24   
#define I2C_SLV0_ADDR    0x25
#define I2C_SLV0_REG     0x26
#define I2C_SLV0_CTRL    0x27
#define I2C_SLV1_ADDR    0x28
#define I2C_SLV1_REG     0x29
#define I2C_SLV1_CTRL    0x2A
#define I2C_SLV2_ADDR    0x2B
#define I2C_SLV2_REG     0x2C
#define I2C_SLV2_CTRL    0x2D
#define I2C_SLV3_ADDR    0x2E
#define I2C_SLV3_REG     0x2F
#define I2C_SLV3_CTRL    0x30
#define I2C_SLV4_ADDR    0x31
#define I2C_SLV4_REG     0x32
#define I2C_SLV4_DO      0x33
#define I2C_SLV4_CTRL    0x34
#define I2C_SLV4_DI      0x35
#define I2C_MST_STATUS   0x36
#define INT_PIN_CFG      0x37
#define INT_ENABLE       0x38
#define DMP_INT_STATUS   0x39  // Check DMP interrupt
#define INT_STATUS       0x3A
#define ACCEL_XOUT_H     0x3B
#define ACCEL_XOUT_L     0x3C
#define ACCEL_YOUT_H     0x3D
#define ACCEL_YOUT_L     0x3E
#define ACCEL_ZOUT_H     0x3F
#define ACCEL_ZOUT_L     0x40
#define TEMP_OUT_H       0x41
#define TEMP_OUT_L       0x42
#define GYRO_XOUT_H      0x43
#define GYRO_XOUT_L      0x44
#define GYRO_YOUT_H      0x45
#define GYRO_YOUT_L      0x46
#define GYRO_ZOUT_H      0x47
#define GYRO_ZOUT_L      0x48
#define EXT_SENS_DATA_00 0x49
#define EXT_SENS_DATA_01 0x4A
#define EXT_SENS_DATA_02 0x4B
#define EXT_SENS_DATA_03 0x4C
#define EXT_SENS_DATA_04 0x4D
#define EXT_SENS_DATA_05 0x4E
#define EXT_SENS_DATA_06 0x4F
#define EXT_SENS_DATA_07 0x50
#define EXT_SENS_DATA_08 0x51
#define EXT_SENS_DATA_09 0x52
#define EXT_SENS_DATA_10 0x53
#define EXT_SENS_DATA_11 0x54
#define EXT_SENS_DATA_12 0x55
#define EXT_SENS_DATA_13 0x56
#define EXT_SENS_DATA_14 0x57
#define EXT_SENS_DATA_15 0x58
#define EXT_SENS_DATA_16 0x59
#define EXT_SENS_DATA_17 0x5A
#define EXT_SENS_DATA_18 0x5B
#define EXT_SENS_DATA_19 0x5C
#define EXT_SENS_DATA_20 0x5D
#define EXT_SENS_DATA_21 0x5E
#define EXT_SENS_DATA_22 0x5F
#define EXT_SENS_DATA_23 0x60
#define MOT_DETECT_STATUS 0x61
#define I2C_SLV0_DO      0x63
#define I2C_SLV1_DO      0x64
#define I2C_SLV2_DO      0x65
#define I2C_SLV3_DO      0x66
#define I2C_MST_DELAY_CTRL 0x67
#define SIGNAL_PATH_RESET  0x68
#define MOT_DETECT_CTRL  0x69
#define USER_CTRL        0x6A  // Bit 7 enable DMP, bit 3 reset DMP
#define PWR_MGMT_1       0x6B // Device defaults to the SLEEP mode
#define PWR_MGMT_2       0x6C
#define DMP_BANK         0x6D  // Activates a specific bank in the DMP
#define DMP_RW_PNT       0x6E  // Set read/write pointer to a specific start address in specified DMP bank
#define DMP_REG          0x6F  // Register in DMP from which to read or to which to write
#define DMP_REG_1        0x70
#define DMP_REG_2        0x71 
#define FIFO_COUNTH      0x72
#define FIFO_COUNTL      0x73
#define FIFO_R_W         0x74
#define WHO_AM_I_MPU6050 0x75 // Should return 0x71
#define XA_OFFSET_H      0x77
#define XA_OFFSET_L      0x78
#define YA_OFFSET_H      0x7A
#define YA_OFFSET_L      0x7B
#define ZA_OFFSET_H      0x7D
#define ZA_OFFSET_L      0x7E


// Using the MSENSR-6050 breakout board, ADO is set to 0 
// Seven-bit device address is 110100 for ADO = 0 and 110101 for ADO = 1
//mbed uses the eight-bit device address, so shift seven-bit addresses left by one!
#define ADO 0
    #if ADO
        #define MPU6050_ADDRESS 0x69<<1  // Device address when ADO = 1
    #else
      #define MPU6050_ADDRESS 0x68<<1  // Device address when ADO = 0
    #endif  

// Set initial input parameters
enum Ascale {
  AFS_2G = 0,
  AFS_4G,
  AFS_8G,
  AFS_16G
};

enum Gscale {
  GFS_250DPS = 0,
  GFS_500DPS,
  GFS_1000DPS,
  GFS_2000DPS
};

enum Mscale {
  MFS_14BITS = 0, // 0.6 uT per LSB
  MFS_16BITS      // 0.15 uT per LSB
};

void MPU6050_INIT(void);
void MPU6050_READ(void);
void writeByte(uint8_t i2c_address , uint8_t reg_address , uint8_t REG_data);
uint8_t readByte(uint8_t address, uint8_t subAddress);
void getMres(void);
void getGres(void);
void getAres(void);
//void readMPU6050_Accel(float * buffer);
//void readMPU6050_Gyro(float * buffer);
void readMPU6050_AccelData(int * buffer);
void readMPU6050_GyroData(int * buffer);
void resetMPU6050(void);

uint8_t Ascale = AFS_2G;     // AFS_2G, AFS_4G, AFS_8G, AFS_16G
uint8_t Gscale = GFS_500DPS; // GFS_250DPS, GFS_500DPS, GFS_1000DPS, GFS_2000DPS
uint8_t Mmode = 0x06;        // Either 8 Hz 0x02) or 100 Hz (0x06) magnetometer data ODR  

int accel_data[3] = {0, 0, 0};  // Stores the 16-bit signed accelerometer sensor output
int gyro_data[3] = {0, 0, 0};   // Stores the 16-bit signed gyro sensor output

uint8_t i2c_package[6] = {0};

void MPU6050_INIT(void)
{
  uint8_t accel_fchoice_b = 1;
  uint8_t A_DLPFCFG = 3;
  uint8_t mpu6050_i = 0;
  // Initialize MPU6050 device
  delay_ms(100);                                          // Delay 100 ms
  writeByte(MPU6050_ADDRESS, PWR_MGMT_1, 0x00);           // Clear sleep mode bit (6), enable all sensors 
  delay_ms(100);                                          // Delay 100 ms for PLL to get established on x-axis gyro; should check for PLL ready interrupt  
  writeByte(MPU6050_ADDRESS , SIGNAL_PATH_RESET , 0x07);
  writeByte(MPU6050_ADDRESS, PWR_MGMT_1, 0x01);           // Set clock source to be PLL with x-axis gyroscope reference, bits 2:0 = 001
  writeByte(MPU6050_ADDRESS, CONFIG, 0x03);  
  writeByte(MPU6050_ADDRESS, SMPLRT_DIV, 0);              // Use a 1000 Hz rate; the same rate set in CONFIG above     
  mpu6050_i = (Gscale << 3) | 0x00;
  writeByte(MPU6050_ADDRESS, GYRO_CONFIG, mpu6050_i);     // Write new GYRO_CONFIG value to register     
  mpu6050_i = (Ascale << 3);
  writeByte(MPU6050_ADDRESS, ACCEL_CONFIG, mpu6050_i);    // Write new ACCEL_CONFIG register value
	mpu6050_i = (0x00 + A_DLPFCFG);	
	writeByte(MPU6050_ADDRESS, ACCEL_CONFIG2, mpu6050_i);   // Write new ACCEL_CONFIG2 register value	
	writeByte(MPU6050_ADDRESS, INT_PIN_CFG, 0x22);    
	writeByte(MPU6050_ADDRESS, INT_ENABLE, 0x00);           // disable data ready (bit 0) interrupt
}


void writeByte(uint8_t address, uint8_t subAddress, uint8_t REG_data)
{
  Single_WriteI2C(address, subAddress, REG_data);
}


uint8_t readByte(uint8_t address, uint8_t subAddress)
{  
    return Single_ReadI2C(address, subAddress); 
}

void readMPU6050_AccelData(int * buffer)
{

  i2c_package[0] = readByte(MPU6050_ADDRESS, ACCEL_XOUT_H);
  i2c_package[1] = readByte(MPU6050_ADDRESS, ACCEL_XOUT_L);
  i2c_package[2] = readByte(MPU6050_ADDRESS, ACCEL_YOUT_H);
  i2c_package[3] = readByte(MPU6050_ADDRESS, ACCEL_YOUT_L);
  i2c_package[4] = readByte(MPU6050_ADDRESS, ACCEL_ZOUT_H);
  i2c_package[5] = readByte(MPU6050_ADDRESS, ACCEL_ZOUT_L);

  buffer[0] = (int)(((int)i2c_package[0] << 8) | i2c_package[1]) ;  // Turn the MSB and LSB into a signed 16-bit value
  buffer[1] = (int)(((int)i2c_package[2] << 8) | i2c_package[3]) ;  
  buffer[2] = (int)(((int)i2c_package[4] << 8) | i2c_package[5]) ; 
}

void readMPU6050_GyroData(int * buffer)
{
  i2c_package[5] = readByte(MPU6050_ADDRESS, GYRO_ZOUT_L);
  i2c_package[4] = readByte(MPU6050_ADDRESS, GYRO_ZOUT_H);
  i2c_package[3] = readByte(MPU6050_ADDRESS, GYRO_YOUT_L);
  i2c_package[2] = readByte(MPU6050_ADDRESS, GYRO_YOUT_H);
  i2c_package[1] = readByte(MPU6050_ADDRESS, GYRO_XOUT_L);
  i2c_package[0] = readByte(MPU6050_ADDRESS, GYRO_XOUT_H);
  buffer[0] = (int)(((int)i2c_package[0] << 8) | i2c_package[1]) ;  // Turn the MSB and LSB into a signed 16-bit value
  buffer[1] = (int)(((int)i2c_package[2] << 8) | i2c_package[3]) ;  
  buffer[2] = (int)(((int)i2c_package[4] << 8) | i2c_package[5]) ; 
}

void resetMPU6050(void) 
{
  // reset device
  writeByte(MPU6050_ADDRESS, PWR_MGMT_1, 0x80); // Write a one to bit 7 reset bit; toggle reset device
  delay_ms(100);
}


#endif
/*End File*/